package App::perlimports;

use strict;
use warnings;

our $VERSION = '0.000026';

1;

# ABSTRACT: Make implicit imports explicit

__END__

=pod

=encoding UTF-8

=head1 NAME

App::perlimports - Make implicit imports explicit

=head1 VERSION

version 0.000026

=head1 DESCRIPTION

This distribution provides the L<perlimports> command line interface (CLI),
which aims to automate the cleanup and maintenance of Perl import statements.
See L<perlimports> for detailed information on how to use this tool.

=head1 AUTHOR

Olaf Alders <olaf@wundercounter.com>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2020 by Olaf Alders.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
